"use strict";

const express = require('express');
const app = express();
const SerialPort = require("serialport");
const config = require('./config.json');
const request = require('request');
const Readline = require('@serialport/parser-readline');
const Delimiter = require('@serialport/parser-delimiter');

let count = 1;
let statusModem = "0K";
let errorMessageModem= "";

//set serialport
const serialPort = new SerialPort(config.core.usb_port, {
    baudRate: 115200
});

const parser = serialPort.pipe(new Readline());

serialPort.on("open", function () {
    console.log('Serial communication open');
    //
    serialPort.write("AT+CNMI?");
    serialPort.write("\r");
    //serialPort.write("AT+CNMI=1,1,2,1,1");
    //serialPort.write("\r");
    console.log('receive sms ON');
});

serialPort.on('error', function(err) {
    statusModem = "NOT OK";
    errorMessageModem = err.message;
    console.log('Error!: ', err.message);
    process.exit(1);
});

parser.on('data', function(data) {
    console.log("incoming data: " + data);
    let lastData = data.toString();
    if (lastData.match(/\+CMTI:/)){
        let numberPattern = /(\d+)/g;
        let slot = lastData.match(numberPattern);
        console.log("slot "+slot);
        console.log("ada message baru");
        readSmsAuto(serialPort, slot);
    } 
});

//function delete message slot ke?
function deleteMessage(serialPort, slot){
    serialPort.write("AT+CMGD="+slot);
    serialPort.write("\r");
    console.log("delete pesan slot "+slot+" done!");
}
//post to end point url
function postIncomingMessage(nomor, pesan){
    console.log("----------------------------post incoming message-------------------------");
    pesan = pesan.replace(/\s+$/, '');
    let encodePesan = encodeURIComponent(pesan);
    let newUrl = config.core.url_post+"?msg="+encodePesan+"&number="+nomor;
    //post to URL end Point
    request(newUrl, function (error, response, body) {
        if (error) {
           console.log("error post! "+nomor+" msg; \""+pesan+"\" errornya; "+error.message);
        } else if (!error && response.statusCode == 200) {
           console.log("success post "+nomor+" msg; \""+pesan+"\" resp; "+body);
        }
    });
}
//sendSms
function sendSms (req, res, next){
    let msg = req.query.msg;
    let number = req.query.number;
    let reqid = req.query.reqid;
    let apiKey = req.query.apikey;
    //apiKey="746573206D6F64656D2074656C6D6F73656C20666C617368";
    if(!reqid||!apiKey){
        console.log("invalid reqid or apikey;msg "+msg+" number "+number);
        res.json({"status": "invalid parameter", "msg": msg, "number": number, "apikey" : apikey, "reqid": reqid});
    return;
    } else if (apiKey !== config.core.api_key){
        console.log("invalid apiKey;msg \""+msg+"\" number \""+number+"\" reqid \""+reqid);
        res.json({"status": "invalid parameter", "msg": msg, "number": number, "apikey" : apikey, "reqid": reqid});
        return;
    }
    console.log("msg "+ msg + " number " +number);
    //----------------
    serialPort.write("AT+CMGF=1");
    serialPort.write('\r');

    sendingSMS(serialPort, msg, "+"+number, reqid);
    console.log('resp; status '+statusModem+" reqid "+reqid);
    //---------------
    res.json({"status": statusModem+" "+errorMessageModem, "msg": msg, "number": number});
}
//send ussd
function sendUSSD (req, res, next){
    let msg = req.query.msg;
    let reqid = req.query.reqid;
    let apiKey = req.query.apikey;
    //apiKey="746573206D6F64656D2074656C6D6F73656C20666C617368";
    if(!reqid||!apiKey){
        console.log("invalid reqid or apikey;msg "+msg);
        res.json({"status": "invalid parameter", "msg": msg, "apikey" : apikey, "reqid": reqid});
        return;
    } else if (apiKey !== config.core.api_key){
        console.log("invalid apiKey;msg \""+msg+"\"reqid \""+reqid);
        res.json({"status": "invalid parameter", "msg": msg, "apikey" : apikey, "reqid": reqid});
        return;
    }
    console.log("msg "+ msg+" - "+reqid);

    count++;
    console.log("count "+ count);
    if (count > 2){
        console.log("modem busy!");
        count--;
        res.json({status: "reject", msg: "modem busy,try a few second"});
        return;
    }	

    //----------------
    serialPort.write("AT+CUSD=1,\""+msg+"\"");
    serialPort.write('\r');

    //parse response USSD
    const parserUSSD = serialPort.pipe(new Delimiter({ delimiter: '\",0'}));
    let respUssd = null;
    parserUSSD.on('data',function(dataUSSD){
    //console.log("incoming datad "+dataD);
    let lastData = dataUSSD.toString();
    console.log("incoming dataUSSD "+lastData);
    respUssd = lastData;
    serialPort.unpipe(parserUSSD);
    });

    console.log('resp; status '+statusModem+" reqid "+reqid);
    //---------------
    let countDown = 8;// max witing response USSD 8 detik
    let waitingRespUssd = setInterval (() => {
        console.log(countDown+" second for reqid "+reqid);
        countDown--;
        if (respUssd || countDown < 2){
            let new_respUssd = null;
            //console.log("response USSD; "+respUssd);
            if (respUssd){
                new_respUssd = respUssd.replace(/^[^ ]+\+CUSD:/, '\+CUSD:');
                console.log('n_resp '+new_respUssd);
            }
            count--;	
            res.json({
                "response_ussd": new_respUssd, 
                "status": statusModem+" "+errorMessageModem, 
                "msg": msg,
                "reqid": reqid
            });
            clearInterval(waitingRespUssd);
        }
    },1000);
}

function readSmsAuto(serialPort, slot){

    serialPort.write("AT+CMGR="+slot);// read only slot 1
    serialPort.write('\r');

    const parserD = serialPort.pipe(new Delimiter({ delimiter: '\r\nOK\r\n' }));
    parserD.on('data',function(dataD){
        //console.log("incoming datad "+dataD);
        let lastData = dataD.toString();
        console.log("incoming datad "+lastData);
        if (lastData.match(/\+CMGR:/)){
            console.log("incoming SMS "+lastData);
            // try to parse dataD
            const lines = lastData.split(/\n/);
            console.log("lines1 "+lines[1]);
            console.log("lines2 "+lines[2]);
            const content = lines[1].split(',');
            console.log("content0 "+content[0]);
            console.log("content1 "+content[1]);
            console.log("content2 "+content[2]);
            console.log("content3 "+content[3]);
            console.log("content4 "+content[4]);
            let nomor = content[1].toString();
            nomor = nomor.replace(/"/g,'');
            nomor = nomor.replace(/\+/g,'');
            let pesan = lines[2].toString();
            console.log("nomor nya "+nomor);
            serialPort.unpipe(parserD);
            postIncomingMessage(nomor, pesan);
        }	
    });

    console.log("pesan slot "+slot+" sudah di baca");

    setTimeout(function(){
        serialPort.write("AT+CMGD="+slot);
        serialPort.write("\r");
        console.log("delete pesan slot "+slot+" done!");
    }, 2000);
}

function readSms(req, res, next){
    serialPort.write("AT+CMGF=1");
    serialPort.write('\r');
    serialPort.write('AT+CMGR=0');// read only slot 1
    serialPort.write('\r');
    res.send('sms readed');
}

function sendingSMS(serial, message, phone_no) {
    if (statusModem === "NOT OK"){
        console.log('send sms gagal');
        return statusModem;
    }
    console.log('number '+phone_no);
    serial.write("AT+CMGS=\"" + phone_no + "\"");
    serial.write('\r');
    serial.write(message); 
    console.log(message);
    serial.write(Buffer([0x1A]));
    serial.write('^z');
}

app.get('/sms', sendSms);
app.get('/ussd', sendUSSD);
app.get('/read', readSms);

app.listen(config.core.node_port, () => console.log(`Example app listening on port ${config.core.node_port}!`));
